<?php
defined('ABSPATH') or die('This page may not be accessed directly.');

class WonderPushAdmin {
  const RESOURCES_VERSION = '1.6.4';
  const MENU_SLUG = 'wonderpush';
  const META_BOX_ID = 'wonderpush_meta_box';
  const SAVE_POST_NONCE_ACTION = 'wonderpush_save_post_nonce_action';
  const SAVE_POST_NONCE_KEY = 'wonderpush_save_post_nonce_key';
  const POST_META_ERROR_MESSAGE = 'wonderpush_error_message';
  const POST_META_INFO_MESSAGE = 'wonderpush_info_message';
  const POST_META_LAST_NOTIFICATION_CONTENT = 'wonderpush_last_notification_content';
  const POST_META_LAST_NOTIFICATION_TIMESTAMP = 'wonderpush_last_notification_timestamp';
  const API_RATE_LIMIT_SECONDS = 3;
  const DEDUPLICATION_SECONDS = 60;

  static function init() {
    if (WonderPushUtils::can_modify_settings()) {
      add_action('admin_menu', array(__CLASS__, 'add_admin_page'));
    }
    if (WonderPushUtils::can_send_notifications()) {
      add_action('admin_init', array( __CLASS__, 'add_post_options' ));
    }

    add_action( 'save_post', array(__CLASS__, 'on_save_post'), 1, 3 );
    add_action( 'transition_post_status', array( __CLASS__, 'on_transition_post_status' ), 10, 3 );

    add_action( 'admin_enqueue_scripts', array( __CLASS__, 'admin_wide_scripts_and_styles' ) );
  }

  public static function add_post_options() {
    $settings = WonderPushSettings::getSettings();
    if ($settings->getDisableSendOnPublish()) return;

    // Add meta box for the "post" post type (default)
    add_meta_box(self::META_BOX_ID,
      'WonderPush Push Notifications',
      array( __CLASS__, 'add_post_html' ),
      'post',
      'side',
      'high');

    // Add meta box for all other post types that are public but not built in to WordPress
    $post_types = get_post_types(array('public'   => true, '_builtin' => false), 'names', 'and' );
    foreach ( $post_types  as $post_type ) {
      add_meta_box(
        self::META_BOX_ID,
        'WonderPush Push Notifications',
        array( __CLASS__, 'add_post_html' ),
        $post_type,
        'side',
        'high'
      );
    }
  }
  public static function add_post_html($post) {
    $post_type = $post->post_type;
    $settings = WonderPushSettings::getSettings();

    // Add an nonce field so we can check for it later.
    wp_nonce_field(self::SAVE_POST_NONCE_ACTION, self::SAVE_POST_NONCE_KEY, true);

    // Our plugin config setting "Automatically send a push notification when I publish a post from the WordPress editor"
    $disable_send_by_default = $settings->getDisableSendByDefaultOnPublish();

    /* This is a scheduled post and the user checked "Send a notification on post publish/update". */
    $send_notification_checked = (get_post_meta($post->ID, 'wonderpush_send_notification', true) == true);
    $send_notification_date = get_post_meta($post->ID, 'wonderpush_send_notification_date', true);
    $send_notification_datetime = $send_notification_date ? WonderPushUtils::date_string_to_datetime($send_notification_date) : null;
    $send_notification_time = get_post_meta($post->ID, 'wonderpush_send_notification_time', true);
    $utm_params = array();
    foreach (WonderPushUtils::utm_parameters() as $utm_parameter) {
      $value = get_post_meta($post->ID, "wonderpush_$utm_parameter", true);
      $value = $value ?: $settings->getUtmParameter($utm_parameter);
      $utm_params[$utm_parameter] = $value;
    }
    $date = new DateTime();
    $dates = array();
    if ($send_notification_datetime && $send_notification_datetime < $date) {
        $dates []= clone $send_notification_datetime;
    }
    $dates []= clone $date;
    for ($i = 1; $i < 10; $i++) {
        $date->add(new DateInterval('P1D'));
        $dates []= clone $date;
    }
    if ($send_notification_datetime && $send_notification_datetime > $date) {
      $dates []= clone $send_notification_datetime;
    }
    $times = array(0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23);

    // We check the checkbox if: setting is enabled on Config page, post type is ONLY "post", and the post has not been published (new posts are status "auto-draft")
    $send_notification = (!$disable_send_by_default
        && $post->post_type == "post"
        &&  in_array($post->post_status, array("future", "draft", "auto-draft", "pending")))
      || $send_notification_checked;
    ?>
      <input type="hidden" name="wonderpush_meta_box_present" value="true" />
      <input type="checkbox" name="send_wonderpush_notification" value="true" <?php if ($send_notification) { echo "checked"; } ?> />
      <label>
        <?php if ($post->post_status == "publish") {
          echo "Send notification on " . $post_type . " update";
        } else {
          echo "Send notification on " . $post_type . " publish";
        } ?>
      </label>
      <br/>
      <h3 style="margin-bottom: 3px;">Send later</h3>
      <small>Schedule the notification to be sent at a later date.</small>
      <br/>
      <select name="wonderpush_send_notification_date">
        <option value=""></option>
        <?php
          foreach ($dates as $date) {
            $date_string = WonderPushUtils::datetime_to_date_string($date);
            ?><option
              <?php echo $date_string === $send_notification_date ? 'selected="selected"' : '' ?>
              value="<?php echo $date_string ?>"><?php echo $date->format('D, d M Y') ?></option><?php
          }
        ?>
      </select>
      at
      <select name="wonderpush_send_notification_time">
          <option value=""></option>
        <?php
          foreach ($times as $time) {
            ?><option
              <?php echo $send_notification_time !== '' && $time == $send_notification_time ? 'selected="selected"' : '' ?>
              value="<?php echo $time ?>"><?php echo sprintf('%02d', $time) ?>h</option><?php
          }
        ?>
      </select>
      <?php echo date_default_timezone_get() ?>
      <h3 style="margin-bottom: 3px;">Google campaign parameters</h3>
      <small>Campaign params help you see push notification traffic in Google Analytics. Learn <a target="_blank" href="https://support.google.com/analytics/answer/1033863#parameters">more</a>.</small>
      <?php foreach (WonderPushUtils::utm_parameters() as $utm_parameter): ?>
          <div style="width: 100%; display: flex; align-items: center; justify-content: flex-start;">
              <span style="min-width: 100px;"><?php echo $utm_parameter ?>:</span>
              <input style="flex-grow: 1; min-width: 100px; display: inline-block;" type="text" name="wonderpush_<?php echo $utm_parameter ?>" value="<?php echo esc_attr(array_key_exists($utm_parameter, $utm_params) ? $utm_params[$utm_parameter] : '') ?>" />
          </div>
      <?php endforeach; ?>
    <?php
  }
  public static function on_save_post($post_id) {
    // Check nonce
    if (!isset( $_POST[self::SAVE_POST_NONCE_KEY] ) ) {
      return $post_id;
    }

    $nonce = $_POST[self::SAVE_POST_NONCE_KEY];

    // Verify nonce
    if (!wp_verify_nonce($nonce, self::SAVE_POST_NONCE_ACTION)) {
      return $post_id;
    }

    if (defined('DOING_AUTOSAVE') && DOING_AUTOSAVE) {
      return $post_id;
    }

    if (array_key_exists('wonderpush_meta_box_present', $_POST)) {
      update_post_meta($post_id, 'wonderpush_meta_box_present', true);
    } else {
      update_post_meta($post_id, 'wonderpush_meta_box_present', false);
    }

    if (array_key_exists('send_wonderpush_notification', $_POST)) {
      update_post_meta($post_id, 'wonderpush_send_notification', true);
    } else {
      update_post_meta($post_id, 'wonderpush_send_notification', false);
    }

    if (array_key_exists('wonderpush_send_notification_date', $_POST)) {
      $meta_value = trim(sanitize_text_field($_POST['wonderpush_send_notification_date']));
      if (WonderPushUtils::is_valid_date_string($meta_value)) {
        update_post_meta($post_id, 'wonderpush_send_notification_date', $meta_value);
      }
    } else {
      update_post_meta($post_id, 'wonderpush_send_notification_date', null);
    }

    if (array_key_exists('wonderpush_send_notification_time', $_POST)) {
      $meta_value = trim(sanitize_text_field($_POST['wonderpush_send_notification_time']));
      if (WonderPushUtils::is_valid_time_string($meta_value)) {
        update_post_meta($post_id, 'wonderpush_send_notification_time', $meta_value);
      }
    } else {
      update_post_meta($post_id, 'wonderpush_send_notification_time', null);
    }

    foreach (WonderPushUtils::utm_parameters() as $utm_parameter) {
      $key = "wonderpush_$utm_parameter";
      if (array_key_exists($key, $_POST)) {
        $meta_value = trim(sanitize_text_field($_POST[$key]));
        update_post_meta($post_id, $key, $meta_value && strlen($meta_value) ? $meta_value : null);
      }
    }
  }
  public static function on_transition_post_status( $new_status, $old_status, $post ) {
    if ($old_status === 'trash' && $new_status === 'publish') {
      return;
    }
    if (!(empty($post) ||
      $new_status !== "publish" ||
      $post->post_type == 'page')) {
      self::send_notification_on_post($new_status, $old_status, $post);
    }
  }

  public static function send_notification_on_post($new_status, $old_status, $post) {
    try {
      if (!WonderPushUtils::is_curl_installed()) {
        return;
      }
      $settings = WonderPushSettings::getSettings();
      $access_token = $settings->getAccessToken();
      if (!$access_token || $settings->getDisableSendOnPublish()) return;

      // Returns true if there is POST data
      $was_posted = !empty($_POST);

      // When this post was created or updated, the meta box in the WordPress post editor screen was visible
      $wonderpush_meta_box_present = $was_posted && array_key_exists('wonderpush_meta_box_present', $_POST) && $_POST['wonderpush_meta_box_present'] === 'true';

      // The checkbox "Send notification on post publish/update" on the meta box is checked
      $wonderpush_meta_box_send_notification_checked = $was_posted && array_key_exists('send_wonderpush_notification', $_POST) && $_POST['send_wonderpush_notification'] === 'true';

      // The notification date was filled
      $wonderpush_meta_send_notification_date = null;
      if ($was_posted && array_key_exists('wonderpush_send_notification_date', $_POST)) {
        $meta_value = trim(sanitize_text_field($_POST['wonderpush_send_notification_date']));
        if (WonderPushUtils::is_valid_date_string($meta_value)) {
          $wonderpush_meta_send_notification_date = $meta_value;
        }
      }

      // The notification time was filled
      $wonderpush_meta_send_notification_time = null;
      if ($was_posted && array_key_exists('wonderpush_send_notification_time', $_POST)) {
        $meta_value = trim(sanitize_text_field($_POST['wonderpush_send_notification_time']));
        if (WonderPushUtils::is_valid_time_string($meta_value)) {
          $wonderpush_meta_send_notification_time = $meta_value;
        }
      }

      // utm parameters
      $utm_params = array();
      foreach (WonderPushUtils::utm_parameters() as $utm_parameter) {
        $value = null;
        $key = "wonderpush_$utm_parameter";
        if ($was_posted) {
          if (array_key_exists($key, $_POST)) {
            $value = $_POST[$key];
          }
        } else {
          $value = get_post_meta($post->ID, $key, true);
        }
        $value = $value ? trim(sanitize_text_field($value)) : $value;
        $value = $value && strlen($value) > 256 ? substr($value, 0, 256) : $value;
        $value = $value && strlen($value) ? $value : null;
        if ($value) $utm_params[$utm_parameter] = $value;
      }
      // This is a scheduled post and the meta box was present.
      $post_metadata_was_wonderpush_meta_box_present = (get_post_meta($post->ID, 'wonderpush_meta_box_present', true) == true);

      // This is a scheduled post and the user checked "Send a notification on post publish/update".
      $post_metadata_was_send_notification_checked = (get_post_meta($post->ID, 'wonderpush_send_notification', true) == true);

      // This is a scheduled post and the user filled notification date
      $post_metadata_send_notification_date = get_post_meta($post->ID, 'wonderpush_send_notification_date', true);

      // This is a scheduled post and the user filled notification time
      $post_metadata_send_notification_time = get_post_meta($post->ID, 'wonderpush_send_notification_time', true);

      // Either we were just posted from the WordPress post editor form, or this is a scheduled notification and it was previously submitted from the post editor form
      $posted_from_wordpress_editor = $wonderpush_meta_box_present || $post_metadata_was_wonderpush_meta_box_present;

      $send_notification_datetime = null;
      if ($posted_from_wordpress_editor) {
        $do_send_notification = ($was_posted && $wonderpush_meta_box_send_notification_checked) ||
          (!$was_posted && $post_metadata_was_send_notification_checked);

        if ($was_posted) {
          if ($wonderpush_meta_send_notification_date) {
            $send_notification_datetime = WonderPushUtils::date_string_to_datetime($wonderpush_meta_send_notification_date);
            if ($send_notification_datetime && $wonderpush_meta_send_notification_time !== null) {
              $send_notification_datetime->setTime($wonderpush_meta_send_notification_time, 0);
            }
          }
        } else {
          if ($post_metadata_send_notification_date) {
            $send_notification_datetime = WonderPushUtils::date_string_to_datetime($post_metadata_send_notification_date);
            if ($send_notification_datetime && $post_metadata_send_notification_time !== null) {
              $send_notification_datetime->setTime($post_metadata_send_notification_time, 0);
            }
          }
        }
        // Discard when in the past:
        if ($send_notification_datetime && $send_notification_datetime <= new DateTime()) {
          $send_notification_datetime = null;
        }
      } else {
        // This was not submitted via the WordPress editor
        $do_send_notification = false;
      }

      if (!$do_send_notification) return;

      // Create WonderPush client
      $wonderPushClient = new \WonderPush\WonderPush($access_token);
      $segmentId = '@ALL';

      update_post_meta($post->ID, 'wonderpush_meta_box_present', false);
      update_post_meta($post->ID, 'wonderpush_send_notification', false);

      // Some WordPress environments seem to be inconsistent about whether on_save_post is called before transition_post_status
      // This sets the metadata back to true, and will cause a post to be sent even if the checkbox is not checked the next time
      // We remove all related $_POST data to prevent this
      if ($was_posted) {
        if (array_key_exists('wonderpush_meta_box_present', $_POST)) {
          unset($_POST['wonderpush_meta_box_present']);
        }
        if (array_key_exists('send_wonderpush_notification', $_POST)) {
          unset($_POST['send_wonderpush_notification']);
        }
      }

      $title = WonderPushUtils::decode_entities(get_the_title($post->ID));

      $site_title = "";
      if ($settings->getNotificationTitle()) {
        $site_title = WonderPushUtils::decode_entities($settings->getNotificationTitle());
      } else {
        $site_title = WonderPushUtils::decode_entities(get_bloginfo('name'));
      }

      $thumbnail_image = null;
      $large_image = null;
      if (has_post_thumbnail($post->ID)) {

        $post_thumbnail_id = get_post_thumbnail_id($post->ID);
        // Higher resolution (2x retina, + a little more) for the notification small icon
        $thumbnail_sized_images_array = wp_get_attachment_image_src($post_thumbnail_id, array(192, 192), true);
        // Much higher resolution for the notification large image
        $large_sized_images_array = wp_get_attachment_image_src($post_thumbnail_id, 'large', true);

        $config_use_featured_image_as_icon = !($settings->getDisableUsePostImageForIcon());
        $config_use_featured_image_as_image = !($settings->getDisableUsePostImageForLargeImage());

        // get the icon image from wordpress if it exists
        if ($config_use_featured_image_as_icon) {
          $thumbnail_image = $thumbnail_sized_images_array[0];
        }
        if ($config_use_featured_image_as_image) {
          $large_image = $large_sized_images_array[0];
        }
      }

      // Send the notification
      $notification = new \WonderPush\Obj\Notification();
      $alert = new \WonderPush\Obj\NotificationAlert();
      $notification->setAlert($alert);
      $permalink = get_permalink($post->ID);
      $target_url = WonderPushUtils::inject_query_string_params($permalink, $utm_params);
      $alert->setTargetUrl($target_url);
      $alert->setTitle($site_title);
      $alert->setText($title);

      // Android
      $android = new \WonderPush\Obj\NotificationAlertAndroid();
      $alert->setAndroid($android);
      if ($large_image) {
        $android->setBigPicture($large_image);
        $android->setType('bigPicture');
      }
      $ios = new \WonderPush\Obj\NotificationAlertIos();
      $alert->setIos($ios);
      if ($large_image) {
        $attachment = new \WonderPush\Obj\NotificationAlertIosAttachment();
        $attachment->setUrl($large_image);
        $attachment->setType('image/png'); // Valid for all image types
        $ios->setAttachments(array($attachment));
      }
      $ios->setSound('default');
      $web = new \WonderPush\Obj\NotificationAlertWeb();
      $alert->setWeb($web);
      if ($thumbnail_image) $web->setIcon($thumbnail_image);
      if ($large_image) $web->setImage($large_image);
      $params = new \WonderPush\Params\DeliveriesCreateParams();
      $params->setNotification($notification);
      $params->setTargetSegmentIds($segmentId);
      if ($send_notification_datetime) {
          $timestamp = $send_notification_datetime->getTimestamp() * 1000;
          $params->setDeliveryDate($timestamp);
      }

      // Deduplicate notifications
      $last_sent_title = get_post_meta($post->ID, self::POST_META_LAST_NOTIFICATION_CONTENT, true);
      $last_sent_timestamp = get_post_meta($post->ID, self::POST_META_LAST_NOTIFICATION_TIMESTAMP, true);
      $elapsed = current_time('timestamp') - ($last_sent_timestamp ? $last_sent_timestamp : 0);
      if ($elapsed < self::DEDUPLICATION_SECONDS && $last_sent_title === $title) {
        if (defined('WP_DEBUG') && WP_DEBUG) {
          error_log('Discarding duplicate notification: ' . json_encode((object)$params->toArray()));
        }
        return;
      }

      // Rate limit
      $wait_time = self::get_sending_rate_limit_wait_time();
      if ($wait_time) {
        update_post_meta($post->ID, self::POST_META_ERROR_MESSAGE, 'You must wait ' . $wait_time . 's before sending another notification');
        return;
      }

      // Remember last notification content and timestamp
      update_post_meta($post->ID, self::POST_META_LAST_NOTIFICATION_CONTENT, $title);
      update_post_meta($post->ID, self::POST_META_LAST_NOTIFICATION_TIMESTAMP, current_time('timestamp'));

      // Send the notification
      if (defined('WP_DEBUG') && WP_DEBUG) {
        error_log('Sending WonderPush notification: ' . json_encode((object)$params->toArray()));
      }
      self::update_last_sent_timestamp();
      $response = $wonderPushClient->deliveries()->create($params);

      // Handle success/failure
      if ($response->isSuccess()) {
        if ($settings->getDisableFeedbackOnPublish()) {
          update_post_meta($post->ID, self::POST_META_INFO_MESSAGE, 'WonderPush notification sent.');
        } else {
          // Fetch the number of subscribers
          $countResponse = $wonderPushClient->installations()->all(array(
            'limit' => 0,
            'reachability' => 'optIn',
            'segmentIds' => $segmentId,
          ));
          $count = $countResponse->getCount();
          if ($count) {
            if ($send_notification_datetime) {
              $formatted_date = $send_notification_datetime->format(DateTime::RFC850);
              update_post_meta($post->ID, self::POST_META_INFO_MESSAGE, "WonderPush will send a notification to {$count} subscribers on {$formatted_date}.");
            } else {
              update_post_meta($post->ID, self::POST_META_INFO_MESSAGE, "WonderPush notification sent to {$count} subscribers.");
            }
          } else {
            update_post_meta($post->ID, self::POST_META_ERROR_MESSAGE, "WonderPush notification sent but you seem to have no subscribers.");
          }
        }
      } else {
        update_post_meta($post->ID, self::POST_META_ERROR_MESSAGE, "WonderPush notification could not be sent.");
      }
    } catch (\WonderPush\Errors\Base $e) {
      switch ($e->getCode()) {
        default:
          update_post_meta($post->ID, self::POST_META_ERROR_MESSAGE, $e->getMessage());
          break;
      }
    } catch (Exception $e) {
      error_log('Caught Exception:' . $e->getMessage());
    }
  }

  public static function admin_wide_scripts_and_styles() {
    wp_register_script('typesafe-actions', WONDERPUSH_PLUGIN_URL . 'assets/js/typesafe-actions.js', false, '4.4.0');
    wp_register_script('redux-observable', WONDERPUSH_PLUGIN_URL . 'assets/js/redux-observable.js', false, '1.1.0');
    wp_register_script('qs', WONDERPUSH_PLUGIN_URL . 'assets/js/qs.js', false, '6.7.0');
    wp_register_script('class-transformer', WONDERPUSH_PLUGIN_URL . 'assets/js/class-transformer.js', false, '0.2.3');

    wp_register_script('axios', WONDERPUSH_PLUGIN_URL . 'assets/js/axios.js', false, '0.19.0');
    wp_register_script('react-redux', WONDERPUSH_PLUGIN_URL . 'assets/js/react-redux.js', array('react','react-dom','redux'), '6.0.1');
    wp_register_script('redux', WONDERPUSH_PLUGIN_URL . 'assets/js/redux.js', false, '4.0.1');
    wp_register_script('reflect-metadata', WONDERPUSH_PLUGIN_URL . 'assets/js/reflect-metadata.js', false, '0.1.13');
    wp_register_script('rxjs', WONDERPUSH_PLUGIN_URL . 'assets/js/rxjs.js', false, '6.5.2');
    wp_register_script('sprintf-js', WONDERPUSH_PLUGIN_URL . 'assets/js/sprintf-js.js', false, '1.1.2');
    wp_enqueue_style('wonderpush-admin-styles', WONDERPUSH_PLUGIN_URL . 'assets/wonderpush-admin.css', false, self::RESOURCES_VERSION);
    wp_enqueue_script('wonderpush-notice', WONDERPUSH_PLUGIN_URL . 'wonderpush-notice.js', false, self::RESOURCES_VERSION);
    wp_localize_script('wonderpush-notice', 'WonderPushNotice', array(
      'nonce' => WonderPushAPI::get_nonce()
    ));
  }

  public static function admin_scripts() {
    wp_enqueue_script('wonderpush-admin-page', WONDERPUSH_PLUGIN_URL . 'wonderpush-admin-page.js', array('react-redux', 'axios', 'rxjs', 'reflect-metadata', 'sprintf-js', 'class-transformer', 'qs', 'redux-observable', 'typesafe-actions'), self::RESOURCES_VERSION);
    wp_localize_script('wonderpush-admin-page', 'WonderPushWordPress', array(
      'pluginUrl' => WONDERPUSH_PLUGIN_URL,
      'ajaxUrl' => admin_url("admin-ajax.php"),
      'siteName' => WonderPushUtils::decode_entities(get_bloginfo('name')),
      'siteUrl' => WonderPushUtils::decode_entities(get_bloginfo('url')),
      'wordPressUrl' => WonderPushUtils::decode_entities(get_bloginfo('wpurl')),
      'wooCommerce' => WonderPushUtils::get_woocommerce() ? true : false,
      'amp' => WonderPushUtils::is_amp_installed() ? true : false,
      'nonce' => WonderPushAPI::get_nonce()
    ));
    wp_enqueue_style('bootstrap', WONDERPUSH_PLUGIN_URL . 'assets/bootstrap.min.css', false, self::RESOURCES_VERSION);
    wp_enqueue_style('wonderpush-admin-page', WONDERPUSH_PLUGIN_URL . 'wonderpush-admin-page.css', false, self::RESOURCES_VERSION);
  }

  public static function add_admin_page() {
    $menu = add_menu_page('WonderPush',
      'WonderPush',
      'manage_options',
      self::MENU_SLUG,
      array(__CLASS__, 'admin_page')
    );
    add_action('load-' . $menu, array(__CLASS__, 'admin_page_load'));
  }

  private static function show_page($page) {
      if (!WonderPushUtils::is_curl_installed()):
        ?>
          <div class="error notice">
              <p>
                  <?php echo __('<a href="https://www.php.net/manual/en/book.curl.php" target="_blank">cURL</a> is not available in this PHP installation, WonderPush cannot operate properly without cURL installed.') ?>
              </p>
          </div>
      <?php
      endif;
    ?>
      <div class="wonderpush-page" id="wonderpush-<?php echo $page ?>"></div>
      <script>
        (function () {
          window.WonderPushShowPage('<?php echo $page ?>', document.getElementById('wonderpush-<?php echo $page ?>'));
        })();
      </script><?php
  }

  public static function admin_page() {
    self::show_page('admin-page');
  }

  public static function admin_page_load() {
    // admin styles
    add_action('admin_enqueue_scripts', array(__CLASS__, 'admin_scripts'));
  }

  public static function get_sending_rate_limit_wait_time() {
    $last_send_time = get_option('wonderpush.last_send_time');
    if ($last_send_time) {
      $current_time = current_time('timestamp');
      $time_elapsed_since_last_send = self::API_RATE_LIMIT_SECONDS - ($current_time - intval($last_send_time));
      if ($time_elapsed_since_last_send > 0) {
        return $time_elapsed_since_last_send;
      }
    }
    return false;
  }

  /**
   * Updates the last sent timestamp, used in rate limiting notifications sent more than 1 per minute.
   */
  public static function update_last_sent_timestamp() {
    $current_time = current_time('timestamp');
    update_option('wonderpush.last_send_time', $current_time);
  }

}
